@@ -168,11 +168,6 @@ test_that('active bindings work', {
   app$port <- 10
   expect_equal(app$port, 10)
 
-  expect_snapshot(app$refresh_rate <- 'test', error = TRUE)
-  expect_snapshot(app$refresh_rate <- 1:5, error = TRUE)
-  app$refresh_rate <- 10.5
-  expect_equal(app$refresh_rate, 10.5)
-
   expect_snapshot(app$refresh_rate_nb <- 'test', error = TRUE)
   expect_snapshot(app$refresh_rate_nb <- 1:5, error = TRUE)
   app$refresh_rate_nb <- 10.5
@@ -183,6 +178,9 @@ test_that('active bindings work', {
   dir <- tempdir()
   app$trigger_dir <- dir
   expect_equal(app$trigger_dir, dir)
+
+  skip_on_cran()
+  lifecycle::expect_deprecated(app$refresh_rate <- 10.5)
 })
 
 test_that('life cycle events get fired', {
@@ -637,7 +635,7 @@ test_that("Logging can be configured", {
   app <- standard_app(FALSE)
   old_format <- app$access_log_format
   app$access_log_format <- combined_log_format
-  expect_equal(app$access_log_format, combined_log_format)
+  expect_equal(app$access_log_format, combined_log_formatter)
   app$on('test', function(server, ...) {
     server$log('test', 'this is a test')
   })
